@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2010-2011 Platform Computing
+# Copyright (C) 2010-2012 Platform Computing
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -144,7 +144,7 @@ IPNETWORKINTERFACE = Mixin('http://schemas.ogf.org/occi/infrastructure/' \
                            [],
                            'L3/L4 capabilities for L2 Network Interface',
                            IPNETWORKINTERFACE_ATTRIBUTES,
-                           '/network/interface/ip')
+                           '/network/interface/ip/')
 
 STORAGELINK_ATTRIBUTES = {'occi.storagelink.deviceid': '',
                           'occi.storagelink.mountpoint': '',
