@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2010-2011 Platform Computing
+# Copyright (C) 2010-2012 Platform Computing
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -23,11 +23,13 @@ Created on Jul 15, 2011
 @author: tmetsch
 '''
 
-# disabling 'Method is abstract' pylint check (HTML only support GET!)
+# disabling 'Method is abstract' pylint check (HTML only support simple ops!)
 # pylint: disable=W0223
 
+from occi.core_model import Category
 from occi.core_model import Resource, Link
-from occi.protocol.occi_rendering import Rendering
+from occi.handlers import QUERY_STRING
+from occi.protocol.rendering import Rendering
 
 
 class HTMLRendering(Rendering):
@@ -55,7 +57,7 @@ class HTMLRendering(Rendering):
             height: 2em; \
            } \
            #header li { \
-            margin: 0.3em 0.5em 0.3em 0.5em; \
+            margin: 0.2em 0.5em 0.3em 0.5em; \
             font-weight: bold; \
             display: inline-block; \
            } \
@@ -66,7 +68,6 @@ class HTMLRendering(Rendering):
            } \
            #header li a:hover { \
             background: #73c167;\
-            border-bottom: 2px solid #73c167; \
            } \
            #breadcrumb { \
             background: #efefea; \
@@ -97,6 +98,15 @@ class HTMLRendering(Rendering):
            a { \
             color: #73c167; \
            } \
+           a:visited { \
+            color: #444; \
+           } \
+           p {\
+            margin: 1em; \
+           } \
+           form {\
+            margin: 1em; \
+           } \
            #entity { \
             margin: 1em; \
            }"
@@ -113,6 +123,58 @@ class HTMLRendering(Rendering):
             self.css = css
 
     def from_entity(self, entity):
+        tmp = self._from_entity_head_html(entity)
+
+        if 'occi.core.id' not in entity.attributes:
+            entity.attributes['occi.core.id'] = entity.identifier
+        if isinstance(entity, Resource):
+            if len(entity.links) > 0:
+                tmp += '\n\t\t\t<h2>Links</h2><table>'
+                tmp += '<tr><th>Kind</th><th>Link</th><th>Target</th></tr>'
+                for item in entity.links:
+                    tmp += '<tr><td>' + item.kind.term + '</td>'
+                    tmp += '<td><a href="' + item.identifier + '">'
+                    tmp += item.identifier + '</a></td>'
+                    tmp += '<td><a href="' + item.target.identifier + '">'
+                    tmp += item.target.identifier + '</a></td>'
+                    tmp += '</tr>'
+                tmp += '</table>\n'
+
+        elif isinstance(entity, Link):
+            tmp += '\n\t\t\t<h2>Source &amp; Target</h2><ul>'
+            tmp += '<li><strong>Source: </strong>'
+            tmp += '<a href="' + entity.source.identifier + '">'
+            tmp += entity.source.identifier + '</a></li>'
+            tmp += '<li><strong>Target: </strong>'
+            tmp += '<a href="' + entity.target.identifier + '">'
+            tmp += entity.target.identifier + '</a></li></ul>\n'
+
+        if len(entity.attributes.keys()) > 0:
+            tmp += '\n\t\t\t<h2>Attributes</h2><table>'
+            for item in entity.attributes.keys():
+                tmp += '<tr><th>' + item + '</th><td>'
+                tmp += str(entity.attributes[item]) + '</td></tr>'
+            tmp += '</table>\n'
+
+        if len(entity.actions) > 0:
+            tmp += '\n\t\t\t<h2>Actions</h2>'
+            for action in entity.actions:
+                tmp += '<FORM action="' + entity.identifier + '?action='
+                tmp += str(action.term) + '?scheme=' + str(action.scheme)
+                tmp += '" method="post" />' '<INPUT type="submit" value="'
+                tmp += str(action.term) + '"/></FORM>'
+            tmp += '\n'
+
+        tmp += '\n\t\t</div>\n\t</body>\n</html>'
+
+        return {'Content-Type': self.mime_type}, tmp
+
+    def _from_entity_head_html(self, entity):
+        '''
+        Private helper function which creates the first part of the HTML.
+
+        entity -- The entity to render.
+        '''
         tmp = '<html>\n\t<head>\n'
         tmp += '\t\t<title>Resource: ' + entity.identifier + '</title>\n'
         tmp += '\t\t<style type="text/css"><!-- ' + self.css + ' --></style>\n'
@@ -128,57 +190,26 @@ class HTMLRendering(Rendering):
         for item in entity.identifier.split('/')[1:-1]:
             path += item + '/'
             tmp += ' <a href="' + path + '">' + item + "</a> / "
-        tmp += entity.identifier.split('/')[-1]
+        tmp += '<a href="' + entity.identifier + '">'
+        tmp += entity.identifier.split('/')[-1] + '</a>'
         tmp += '</div>\n'
 
         # body
-        tmp += '\t\t<div id="entity">\n'
-        tmp += '\t\t\t<h2>Kind</h2><ul><li>'
-        tmp += str(entity.kind) + '</li></ul>\n'
+        tmp += '\t\t<div id="entity">\n\t\t\t<h2>Kind</h2><ul><li>'
+        tmp += '<a href="/-/#' + entity.kind.scheme + '-'
+        tmp += entity.kind.term + '">' + str(entity.kind) + '</a></li></ul>\n'
         if len(entity.mixins) > 0:
             tmp += '\t\t\t<h2>Mixins</h2><ul>'
             for item in entity.mixins:
-                tmp += '<li>' + str(item) + '</li>'
+                tmp += '<li><a href="/-/#' + item.scheme + '-'
+                tmp += item.term + '">' + str(item) + '</a></li>'
             tmp += '</ul>\n'
 
-        entity.attributes['occi.core.id'] = entity.identifier
-        if isinstance(entity, Resource):
-            if len(entity.links) > 0:
-                tmp += '\t\t\t<h2>Links</h2><table>'
-                tmp += '<tr><th>Kind</th><th>Link</th><th>Target</th></tr>'
-                for item in entity.links:
-                    tmp += '<tr><td>' + item.kind.term + '</td>'
-                    tmp += '<td><a href="' + item.identifier + '">'
-                    tmp += item.identifier + '</a></td>'
-                    tmp += '<td><a href="' + item.target.identifier + '">'
-                    tmp += item.target.identifier + '</a></td>'
-                    tmp += '</tr>'
-                tmp += '</table>\n'
-
-        elif isinstance(entity, Link):
-            tmp += '\t\t\t<h2>Source &amp; Target</h2><ul>'
-            tmp += '<li><strong>Source: </strong>'
-            tmp += '<a href="' + entity.source.identifier + '">'
-            tmp += entity.source.identifier + '</a></li>'
-            tmp += '<li><strong>Target: </strong>'
-            tmp += '<a href="' + entity.target.identifier + '">'
-            tmp += entity.target.identifier + '</a></li></ul>\n'
-
-        if len(entity.attributes.keys()) > 0:
-            tmp += '\t\t\t<h2>Attributes</h2><table>'
-            for item in entity.attributes.keys():
-                tmp += '<tr><th>' + item + '</th><td>'
-                tmp += str(entity.attributes[item]) + '</td></tr>'
-            tmp += '</table>\n'
-
-        if len(entity.actions) > 0:
-            tmp += '\t\t\t<h2>Actions</h2><ul>'
-            for action in entity.actions:
-                tmp += '<li>' + str(action.term) + '</li>'
-            tmp += '<ul>\n'
-
-        tmp += '\t\t</div>\n\t</body>\n</html>'
-        return {}, tmp
+        if hasattr(entity, 'summary') and entity.summary != '':
+            tmp += '\t\t\t<h2>Summary</h2><p>' + str(entity.summary) + '</p>\n'
+        if entity.title != '':
+            tmp += '\t\t\t<h2>Title</h2><p>' + str(entity.title) + '</p>\n'
+        return tmp
 
     def from_entities(self, entities, key):
         tmp = '<html>\n\t<head>\n'
@@ -208,7 +239,7 @@ class HTMLRendering(Rendering):
             tmp += item.identifier + '</a></li>\n'
         tmp += '\t\t</ul></div>\n'
         tmp += '\t</body>\n</html>'
-        return {}, tmp
+        return {'Content-Type': self.mime_type}, tmp
 
     def from_categories(self, categories):
         tmp = '<html>\n\t<head>\n'
@@ -226,7 +257,8 @@ class HTMLRendering(Rendering):
 
         # body
         for cat in categories:
-            tmp += '\t\t<h2>' + repr(cat).upper() + ': ' + cat.term + '</h2>\n'
+            tmp += '\t\t<h2><a name="' + cat.scheme + '-' + cat.term + '">'
+            tmp += repr(cat).upper() + ': ' + cat.term + '</a></h2>\n'
             tmp += '\t\t<table>\n'
             tmp += '\t\t\t<tr><th>Scheme</th><td>' + cat.scheme
             tmp += '</td></tr>\n'
@@ -266,4 +298,28 @@ class HTMLRendering(Rendering):
             tmp += '\t\t</table>\n'
 
         tmp += '\t</body>\n</html>'
-        return {}, tmp
+        return {'Content-Type': self.mime_type}, tmp
+
+    def to_action(self, headers, body):
+        cat_str = headers.get(QUERY_STRING)
+        scheme_index = cat_str.find('scheme=')
+        if scheme_index != -1:
+            scheme = cat_str[scheme_index + 7:]
+        else:
+            raise AttributeError('Unable to find the scheme. Broken request?.')
+
+        index = cat_str.find('action=')
+        if index != -1:
+            term = cat_str[index + 7:scheme_index - 1]
+        else:
+            raise AttributeError('Unable to find the term. Broken request?.')
+
+        tmp = Category(scheme + '#', term, '', {}, '')
+        cats = self.registry.get_categories()
+        if tmp in cats:
+            for item in self.registry.get_categories():
+                if tmp == item:
+                    del(tmp)
+                    return item
+        else:
+            raise AttributeError('Action is not defined. Check the QI.')
