@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2010-2011 Platform Computing
+# Copyright (C) 2010-2012 Platform Computing
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -23,7 +23,7 @@ Created on Jun 28, 2011
 @author: tmetsch
 '''
 
-from occi.core_model import Category, Link, Mixin
+from occi.core_model import Category, Link, Mixin, Kind
 
 #==============================================================================
 # Following are text/occi and text/plain related parsing functions.
@@ -103,6 +103,28 @@ def get_category_str(category):
     return tmp
 
 
+def _get_link_categories(categories, registry):
+    '''
+    Determine the kind ans mixins for inline link creation.
+
+    categories -- String with a set of category string definitions.
+    registry -- Registry used for this call.
+    '''
+    tmp_kind = None
+    tmp_mixins = []
+    for tmp_cat in categories.split(' '):
+        tempus = tmp_cat.split('#')
+        link_category = get_category(tempus[1].strip() + ';scheme="'
+                                     + tempus[0].strip() + '#"',
+                                     registry.get_categories())
+        if isinstance(link_category, Kind):
+            tmp_kind = link_category
+        else:
+            tmp_mixins.append(link_category)
+
+    return tmp_kind, tmp_mixins
+
+
 def get_link(link_string, source, registry):
     '''
     Create a Link from a string rendering.
@@ -125,10 +147,11 @@ def get_link(link_string, source, registry):
         tmp_category = find_in_string(link_string, 'category')
     except AttributeError:
         raise AttributeError('Could not determine the Category of the Link.')
-    tempus = tmp_category.split('#')
-    link_category = get_category(tempus[1].strip() + ';scheme="'
-                                 + tempus[0].strip() + '#"',
-                                 registry.get_categories())
+
+    tmp_kind, tmp_mixins = _get_link_categories(tmp_category, registry)
+
+    if tmp_kind is None:
+        raise AttributeError('Unable to find the Kind of the Link.')
 
     attributes = {}
     attr_begin = link_string.find('category="') + 12 + len(tmp_category)
@@ -146,7 +169,7 @@ def get_link(link_string, source, registry):
         raise AttributeError('The target for the link cannot be found: '
                              + target_id)
 
-    link = Link(link_id, link_category, [], source, target)
+    link = Link(link_id, tmp_kind, tmp_mixins, source, target)
     link.attributes = attributes
     return link
 
@@ -160,7 +183,13 @@ def get_link_str(link):
     tmp = '<' + link.target.identifier + '>'
     tmp = tmp + '; rel="' + str(link.target.kind) + '"'
     tmp = tmp + '; self="' + str(link.identifier) + '"'
-    tmp = tmp + '; category="' + str(link.kind) + '"'
+    tmp = tmp + '; category="' + str(link.kind)
+
+    if len(link.mixins) > 0:
+        for mixin in link.mixins:
+            tmp = tmp + ' ' + str(mixin)
+
+    tmp = tmp + '"'
 
     link.attributes['occi.core.id'] = link.identifier
     link.attributes['occi.core.source'] = link.source.identifier
@@ -182,7 +211,7 @@ def get_attributes(attribute_string):
         tmp = _strip_all(attribute_string)
         key = _strip_all(tmp.split('=')[0])
         value = tmp.split('=')[1]
-        if value.find('"') is not - 1:
+        if value.find('"') is not -1:
             value = _strip_all(value)
     except IndexError:
         raise AttributeError('Could not determine the given attributes')
