@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2010-2011 Platform Computing
+# Copyright (C) 2010-2012 Platform Computing
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -24,6 +24,8 @@ Created on Jun 28, 2011
 '''
 
 from occi.core_model import Resource, Link
+from occi.handlers import CATEGORY, ATTRIBUTE, LOCATION, LINK, CONTENT_TYPE
+from occi.protocol.rendering import Rendering
 import occi.protocol.occi_parser as parser
 import shlex
 
@@ -43,149 +45,11 @@ class HTTPData(object):
         self.attributes = []
         self.locations = []
 
-
-class Rendering(object):
-    '''
-    All renderings should derive from this class.
-    '''
-
-    def __init__(self, registry):
-        '''
-        Constructor.
-
-        registry -- The registry used for this process.
-        '''
-        self.registry = registry
-
-    def to_entity(self, headers, body, def_kind):
-        '''
-        Given the HTTP headers and the body this method will convert the HTTP
-        data into an entity representation. Must return Resource or Link
-        instances.
-
-        headers -- The HTTP headers.
-        body -- The HTTP body.
-        def_kind -- If provided this kind is taken (Needed for update).
-        '''
-        raise NotImplementedError()
-
-    def from_entity(self, entity):
-        '''
-        Given an entity it will return a HTTP body an header.
-
-        If it's a link make sure source, target attributes are set. If it's a
-        Resource make sure Links are presented properly.
-
-        entity -- The entity which is to rendered.
-        '''
-        raise NotImplementedError()
-
-    def to_entities(self, headers, body):
-        '''
-        Given the HTTP headers and the body this method will convert the HTTP
-        data into a set of entity representations. Must return a set of
-        Resource or Link instances.
-
-        headers -- The HTTP headers.
-        body -- The HTTP body.
-        '''
-        raise NotImplementedError()
-
-    def from_entities(self, entities, key):
-        '''
-        Given an set of entities it will return a HTTP body an header.
-
-        entities -- The entities which will be rendered.
-        key -- Needed for uri-list (see RFC) and html rendering.
-        '''
-        raise NotImplementedError()
-
-    def from_categories(self, categories):
-        '''
-        Given an set of categories it will return a HTTP body an header.
-
-        categories -- The list of categories which is to be rendered.
-        '''
-        raise NotImplementedError()
-
-    def to_action(self, headers, body):
-        '''
-        Given the HTTP headers and the body this method will convert the HTTP
-        data into an Action.
-
-        headers -- The HTTP headers.
-        body -- The HTTP body.
-        '''
-        raise NotImplementedError()
-
-    def to_mixins(self, headers, body):
-        '''
-        Given the HTTP headers and the body this method will convert the HTTP
-        data into a Mixins. Must return a list with Mixin instances.
-
-        headers -- The HTTP headers.
-        body -- The HTTP body.
-        '''
-        raise NotImplementedError()
-
-    def get_filters(self, headers, body):
-        '''
-        Given the HTTP headers and the body this method will convert the HTTP
-        data into a list of categories and attributes.
-
-        headers -- The HTTP headers.
-        body -- The HTTP body.
-        '''
-        raise NotImplementedError()
-
 #==============================================================================
 # text/occi rendering
 #==============================================================================
 
 
-def _extract_data_from_headers(headers):
-    '''
-    Simple method to split out the information from the HTTP headers.
-
-    headers -- The HTTP headers.
-    '''
-    # split out the information
-    data = HTTPData()
-    if 'Category' in headers.keys():
-        data.categories = headers['Category'].split(',')
-    if 'X-OCCI-Attribute' in headers.keys():
-        split = shlex.shlex(headers['X-OCCI-Attribute'], posix=True)
-        split.whitespace += ','
-        split.whitespace_split = True
-        data.attributes = list(split)
-    if 'X-OCCI-Location' in headers.keys():
-        data.locations = headers['X-OCCI-Location'].split(',')
-    if 'Link' in headers.keys():
-        data.links = headers['Link'].split(',')
-    return data
-
-
-def _set_data_to_headers(data):
-    '''
-    Simple method to set all information in the HTTP header.
-
-    data -- The data to set.
-    '''
-    headers = {}
-    body = 'OK'
-
-    if len(data.categories) > 0:
-        headers['Category'] = ', '.join(data.categories)
-    if len(data.links) > 0:
-        headers['Link'] = ', '.join(data.links)
-    if len(data.locations) > 0:
-        headers['X-OCCI-Location'] = ', '.join(data.locations)
-    if len(data.attributes) > 0:
-        headers['X-OCCI-Attribute'] = ', '.join(data.attributes)
-
-    return headers, body
-
-
 def _to_entity(data, def_kind, registry):
     '''
     Extract an entity from the HTTP data object.
@@ -273,7 +137,8 @@ def _from_entity(entity):
     data.categories = cat_str_list
 
     attributes = []
-    entity.attributes['occi.core.id'] = entity.identifier
+    if 'occi.core.id' not in entity.attributes:
+        entity.attributes['occi.core.id'] = entity.identifier
 
     link_str_list = []
     # actions
@@ -402,6 +267,54 @@ def _get_filter(data, registry):
     return categories, attributes
 
 
+def _extract_data_from_headers(headers):
+    '''
+    Simple method to split out the information from the HTTP headers.
+
+    headers -- The HTTP headers.
+    '''
+    # split out the information
+    data = HTTPData()
+    if CATEGORY in headers.keys():
+        data.categories = headers[CATEGORY].split(',')
+    if ATTRIBUTE in headers.keys():
+        split = shlex.shlex(headers[ATTRIBUTE], posix=True)
+        split.whitespace = ','
+        split.whitespace_split = True
+        data.attributes = list(split)
+    if LOCATION in headers.keys():
+        data.locations = headers[LOCATION].split(',')
+    if LINK in headers.keys():
+        data.links = headers[LINK].split(',')
+    return data
+
+
+def _set_data_to_headers(data, mime_type):
+    '''
+    Simple method to set all information in the HTTP header.
+
+    data -- The data to set.
+    mime_type -- The content type to set.
+    '''
+    headers = {}
+    body = 'OK'
+
+    # We're using different header names here - WSGI will take care of correct
+    # 'translation'
+
+    if len(data.categories) > 0:
+        headers[CATEGORY] = ', '.join(data.categories)
+    if len(data.links) > 0:
+        headers[LINK] = ', '.join(data.links)
+    if len(data.locations) > 0:
+        headers[LOCATION] = ', '.join(data.locations)
+    if len(data.attributes) > 0:
+        headers[ATTRIBUTE] = ', '.join(data.attributes)
+    headers[CONTENT_TYPE] = mime_type
+
+    return headers, body
+
+
 class TextOcciRendering(Rendering):
     '''
     This is a rendering which will use the HTTP header to place the information
@@ -429,7 +342,7 @@ class TextOcciRendering(Rendering):
 
         data -- An HTTPData object.
         '''
-        return _set_data_to_headers(data)
+        return _set_data_to_headers(data, self.mime_type)
 
     def to_entity(self, headers, body, def_kind):
         data = self.get_data(headers, body)
@@ -480,14 +393,14 @@ def _extract_data_from_body(body):
     '''
     data = HTTPData()
     for entry in body.split('\n'):
-        if entry.find('Category:') > -1:
-            data.categories.extend(_extract_values(entry, 'Category:'))
-        if entry.find('X-OCCI-Attribute:') > -1:
-            data.attributes.extend(_extract_values(entry, 'X-OCCI-Attribute:'))
-        if entry.find('Link:') > -1:
-            data.links.extend(_extract_values(entry, 'Link:'))
-        if entry.find('X-OCCI-Location:') > -1:
-            data.locations.extend(_extract_values(entry, 'X-OCCI-Location:'))
+        if entry.find(CATEGORY + ':') > -1:
+            data.categories.extend(_extract_values(entry, CATEGORY + ':'))
+        if entry.find(ATTRIBUTE + ':') > -1:
+            data.attributes.extend(_extract_values(entry, ATTRIBUTE + ':'))
+        if entry.find(LINK + ':') > -1:
+            data.links.extend(_extract_values(entry, LINK + ':'))
+        if entry.find(LOCATION + ':') > -1:
+            data.locations.extend(_extract_values(entry, LOCATION + ':'))
     return data
 
 
@@ -504,36 +417,37 @@ def _extract_values(entry, key):
         items.append(tmp)
     else:
         split = shlex.shlex(tmp, posix=True)
-        split.whitespace += ','
+        split.whitespace = ','
         split.whitespace_split = True
         items.extend(list(split))
     return items
 
 
-def _set_data_to_body(data):
+def _set_data_to_body(data, mime_type):
     '''
     Simple method to set all information in the HTTP body.
 
     data -- The data to set.
+    mime_type -- The content type to set.
     '''
     body = ''
     if len(data.categories) > 0:
         for cat in data.categories:
-            body += '\nCategory: ' + cat
+            body += '\n' + CATEGORY + ': ' + cat
 
     if len(data.links) > 0:
         for link in data.links:
-            body += '\nLink: ' + link
+            body += '\n' + LINK + ': ' + link
 
     if len(data.attributes) > 0:
         for attr in data.attributes:
-            body += '\nX-OCCI-Attribute: ' + attr
+            body += '\n' + ATTRIBUTE + ': ' + attr
 
     if len(data.locations) > 0:
         for loc in data.locations:
-            body += '\nX-OCCI-Location: ' + loc
+            body += '\n' + LOCATION + ': ' + loc
 
-    return {}, body
+    return {CONTENT_TYPE: mime_type}, body
 
 
 class TextPlainRendering(TextOcciRendering):
@@ -545,7 +459,7 @@ class TextPlainRendering(TextOcciRendering):
     mime_type = 'text/plain'
 
     def set_data(self, data):
-        return _set_data_to_body(data)
+        return _set_data_to_body(data, self.mime_type)
 
     def get_data(self, headers, body):
         return _extract_data_from_body(body)
@@ -573,7 +487,7 @@ class TextUriListRendering(Rendering):
         body = '# uri:' + str(key)
         for entity in entities:
             body += '\n' + self.registry.get_hostname() + entity.identifier
-        return {}, body
+        return {CONTENT_TYPE: self.mime_type}, body
 
     def from_categories(self, categories):
         raise AttributeError(self.error)
