@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2010-2011 Platform Computing
+# Copyright (C) 2010-2012 Platform Computing
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -25,7 +25,7 @@ Created on Jun 30, 2011
 
 from occi.backend import UserDefinedMixinBackend
 from occi.core_model import Resource, Link, Mixin
-from tornado.web import HTTPError
+from occi.exceptions import HTTPError
 import uuid
 
 #==============================================================================
@@ -33,7 +33,7 @@ import uuid
 #==============================================================================
 
 
-def create_entity(key, entity, registry):
+def create_entity(key, entity, registry, extras):
     '''
     Handles all the model magic during creation of an entity.
 
@@ -44,6 +44,7 @@ def create_entity(key, entity, registry):
     key -- The key for the entity.
     entity -- The entity itself - either Link or Resource instance.
     registry -- The registry used for this process.
+    extras -- Any extra arguments which are defined by the user.
     '''
     entity.identifier = key
 
@@ -57,7 +58,7 @@ def create_entity(key, entity, registry):
                 raise AttributeError('A link with that id is already present')
 
             for back in registry.get_all_backends(link):
-                back.create(link)
+                back.create(link, extras)
 
             registry.add_resource(link.identifier, link)
     elif isinstance(entity, Link):
@@ -66,12 +67,12 @@ def create_entity(key, entity, registry):
     # call all the backends who are associated with this entity.kind...
     backends = registry.get_all_backends(entity)
     for backend in backends:
-        backend.create(entity)
+        backend.create(entity, extras)
 
     registry.add_resource(key, entity)
 
 
-def delete_entity(entity, registry):
+def delete_entity(entity, registry, extras):
     '''
     Handles all the model magic during deletion if an entity.
 
@@ -79,25 +80,27 @@ def delete_entity(entity, registry):
 
     entity -- The entity itself - either Link or Resource instance.
     registry -- The registry used for this process.
+    extras -- Any extra arguments which are defined by the user.
     '''
     if isinstance(entity, Resource):
         # it's an resource - so delete all it's links
         for link in entity.links:
             for back in registry.get_all_backends(link):
-                back.delete(link)
+                back.delete(link, extras)
             registry.delete_resource(link.identifier)
     elif isinstance(entity, Link):
         entity.source.links.remove(entity)
 
     # call all the backends who are associated with this entity.kind...
     backends = registry.get_all_backends(entity)
+
     for backend in backends:
-        backend.delete(entity)
+        backend.delete(entity, extras)
 
     registry.delete_resource(entity.identifier)
 
 
-def replace_entity(old, new, registry):
+def replace_entity(old, new, registry, extras):
     '''
     Replace an entity - backends decide what is done.
 
@@ -106,6 +109,7 @@ def replace_entity(old, new, registry):
     old -- The old entity.
     new -- The new entity.
     registry -- The registry used for this process.
+    extras -- Any extra arguments which are defined by the user.
     '''
     if isinstance(new, Resource) and len(new.links) is not 0:
         raise HTTPError(400, 'It is not recommend to have links in a full' +
@@ -118,11 +122,11 @@ def replace_entity(old, new, registry):
     # call all the backends who are associated with this entity.kind...
     backends = registry.get_all_backends(old)
     for backend in backends:
-        backend.replace(old, new)
+        backend.replace(old, new, extras)
     del(new)
 
 
-def update_entity(old, new, registry):
+def update_entity(old, new, registry, extras):
     '''
     Update an entity - backends decide what is done.
 
@@ -131,6 +135,7 @@ def update_entity(old, new, registry):
     old -- The old entity.
     new -- The new entity.
     registry -- The registry used for this process.
+    extras -- Any extra arguments which are defined by the user.
     '''
     if isinstance(new, Resource) and len(new.links) is not 0:
         raise HTTPError(400, 'It is not recommend to have links in a full' +
@@ -139,11 +144,11 @@ def update_entity(old, new, registry):
     # call all the backends who are associated with this entity.kind...
     backends = registry.get_all_backends(old)
     for backend in backends:
-        backend.update(old, new)
+        backend.update(old, new, extras)
     del(new)
 
 
-def retrieve_entity(entity, registry):
+def retrieve_entity(entity, registry, extras):
     '''
     Retrieves/refreshed an entity.
 
@@ -151,36 +156,38 @@ def retrieve_entity(entity, registry):
 
     entity -- The entity which is to be retrieved.
     registry -- The registry used for this process.
+    extras -- Any extra arguments which are defined by the user.
     '''
     if isinstance(entity, Resource):
         # if it's a resource - retrieve all links...
         for link in entity.links:
             for back in registry.get_all_backends(link):
-                back.retrieve(link)
+                back.retrieve(link, extras)
 
     # call all the backends who are associated with this entity.kind...
     backends = registry.get_all_backends(entity)
     for backend in backends:
-        backend.retrieve(entity)
+        backend.retrieve(entity, extras)
 
 
-def action_entity(entity, action, registry):
+def action_entity(entity, action, registry, extras):
     '''
     Performs an action on the entity.
 
     entity -- The entity on which to perform the operation.
     action -- The action definition.
     registry -- The registry used for this process.
+    extras -- Any extra arguments which are defined by the user.
     '''
     backend = registry.get_backend(action)
-    backend.action(entity, action)
+    backend.action(entity, action, extras)
 
 #==============================================================================
 # Collections
 #==============================================================================
 
 
-def update_collection(mixin, old_entities, new_entities, registry):
+def update_collection(mixin, old_entities, new_entities, registry, extras):
     '''
     Updates a Collection of Mixin. If not present in the current collections
     entities will be added to the collection (aka. assigned the Mixin).
@@ -189,6 +196,7 @@ def update_collection(mixin, old_entities, new_entities, registry):
     old_entities -- The entities which are in the collection to date.
     new_entities -- The entities which should be added to the collection.
     registry -- The registry used for this process.
+    extras -- Any extra arguments which are defined by the user.
     '''
     if not isinstance(mixin, Mixin):
         raise AttributeError('This operation is only supported on Collections'
@@ -196,11 +204,11 @@ def update_collection(mixin, old_entities, new_entities, registry):
     for entity in unique(new_entities, old_entities):
         entity.mixins.append(mixin)
         backend = registry.get_backend(mixin)
-        backend.create(entity)
+        backend.create(entity, extras)
     del(new_entities)
 
 
-def replace_collection(mixin, old_entities, new_entities, registry):
+def replace_collection(mixin, old_entities, new_entities, registry, extras):
     '''
     Replaces a Collection of Mixin. If not present in the current collections
     entities will be added to the collection (aka. assigned the Mixin). If old
@@ -211,6 +219,7 @@ def replace_collection(mixin, old_entities, new_entities, registry):
     old_entities -- The entities which are in the collection to date.
     new_entities -- The new collection of entities.
     registry -- The registry used for this process.
+    extras -- Any extra arguments which are defined by the user.
     '''
     if not isinstance(mixin, Mixin):
         raise AttributeError('This operation is only supported on Collections'
@@ -218,21 +227,22 @@ def replace_collection(mixin, old_entities, new_entities, registry):
     for entity in unique(new_entities, old_entities):
         entity.mixins.append(mixin)
         backend = registry.get_backend(mixin)
-        backend.create(entity)
+        backend.create(entity, extras)
     for entity in unique(old_entities, new_entities):
         backend = registry.get_backend(mixin)
-        backend.delete(entity)
+        backend.delete(entity, extras)
         entity.mixins.remove(mixin)
     del(new_entities)
 
 
-def delete_from_collection(mixin, entities, registry):
+def delete_from_collection(mixin, entities, registry, extras):
     '''
     Removes entities from a collection by removing the mixin from their list.
 
     mixin -- The mixin which defines the collection.
     entities -- The entities which are to be removed.
     registry -- The registry used for this process.
+    extras -- Any extra arguments which are defined by the user.
     '''
     if not isinstance(mixin, Mixin):
         raise AttributeError('This operation is only supported on Collections'
@@ -240,7 +250,7 @@ def delete_from_collection(mixin, entities, registry):
 
     for entity in intersect(entities, registry.get_resources()):
         backend = registry.get_backend(mixin)
-        backend.delete(entity)
+        backend.delete(entity, extras)
         entity.mixins.remove(mixin)
 
 
